FUNCTION splice_in,la,fa,ea,lb,fb,eb,l0,l1,MODE=mode

; version 4.1
;  4 Apr 02 - to calculate errors from spec./phot. error vectors (v 4.1)
; 29 Mar 02 - to return errors in flux units (v 4.0)
;  7 Mar 02 - for additive correction, now returns ratio for error (v 3.0)
;  4 Mar 02 - diagnostic changes
; 30 Jan 02 - cosmetic changes only
; 24 Jan 02 - fix uncertainty calculation (KK)
; 18 Dec 01 - calculate uncertainty in normalization (KK)
;  2 Oct 01 - support very long vectors
;  8 Aug 01 - to correctly check for data-in-range problems
;  7 Aug 01 - to write error message to log file
; 23 Jul 01 - to handle non-standard 1-D array format
;             find multiplicative or additive correction
;             and check for no data in range
;             no longer applies normalization to data
; 20 Jul 01 - to return normalization constant
; 20 Oct 97 - created as splice_inx.pro
;
; finds multiplicative or additive factor to splice two spectral segments
; fits second to first
; uses averages over input wavelength range to determine
;    multiplicative or additive correction
;    (i.e. combines functions of splice_inx.pro and splice_ina.pro
; input:
;	la,lb = spectral wavelengths
;	fa,fb = spectral fluxes (must be same length as la,lb)
;       ea,eb = error vectors accompanying fluxes (same length as la,lb)
;	l0 = starting wavelength
;	l1 = stopping wavelength
;       mode = 0 for multiplicative correction (default)
;       mode = 1 for additive correction
; output:  array of two numbers
;       first is the normalization factor (additive or muliplicative)
;       second is the error in the first as a fraction of normalization if 
;         multiplicative or in flux units if additve
;
; check mode

if (n_elements(mode) eq 0) then mode=0 ; (multiplicative default)

; if l0 > l1, flip them

if (l0 gt l1) then begin
  temp = l0
  l0   = l1
  l1   = temp
endif

; find overlap between segments and determine error condition 
;   error=0 means overlap between la and lb covers full range l0-l1
;   error=1 means la and lb overlap, but not over full range l0-l1
;           correct for this by reducing range l0-l1 to match
;   error=2 means that la and lb do not overlap in any part of range l0-l1

if (min(la) lt min(lb)) then olapmin=min(lb) else olapmin=min(la)
if (max(la) lt max(lb)) then olapmax=max(la) else olapmax=max(lb)

if (olapmax le olapmin) then begin               ; segments don't overlap 
  error=2
endif else begin                                 ; overlap possibilities
  case 1 of
    (olapmin gt l1 or olapmax lt l0)  : error=2  ; no overlap with l0-l1
    (olapmin le l0 and olapmax ge l1) : error=0  ; everything is fine
    else                              : error=1  ; must move l0 and/or l1
  endcase
endelse

; if error=1 then shift l0 and/or l1 and print warning message

if (error eq 1) then begin
  if (olapmin gt l0) then l0=olapmin             ; move l0 up
  if (olapmax lt l1) then l1=olapmax             ; move l1 down
  print,format='("splice_in performed over reduced range   ",f6.3," - ",f6.3," um")',l0,l1
  printf,99,format='("  splice_in performed over reduced range   ",f6.3," - ",f6.3," um")',l0,l1
endif

if (error lt 2) then begin

; set stops

  jastop=long(n_elements(la)-1) ; OK only if la is 1-D
  jbstop=long(n_elements(lb)-1) ; ditto
  j = long(0) & while (la(j) lt l0) do j=j+1 & ja0 = j
  j = long(0) & while (lb(j) lt l0) do j=j+1 & jb0 = j
  j = jastop & while (la(j) gt l1) do j=j-1 & ja1 = j
  j = jbstop & while (lb(j) gt l1) do j=j-1 & jb1 = j

; find the average flux from each band in the overlap region

  mfa = total(fa(ja0:ja1))/float(ja1-ja0+1)
  mfb = total(fb(jb0:jb1))/float(jb1-jb0+1)

; compute uncertainty in the mean using error vectors

  mea = sqrt( 1.0 / total(1.0/ea(ja0:ja1)^2) )
  meb = sqrt( 1.0 / total(1.0/eb(jb0:jb1)^2) )

; compute correction and error

  c=fltarr(2)
  if (mode eq 0) then begin
    c(0) = mfa/mfb 
    c(1) = sqrt( (mea/mfb)^2 + (meb*mfa/mfb^2)^2 )
  endif else if (mode eq 1) then begin
    c(0) = mfa - mfb
    c(1) = sqrt( mea^2 + meb^2)
  endif

endif else begin 

  print,format='("unable to perform splice_in over range   ",f6.3," - ",f6.3," um")',l0,l1
  printf,99,format='("  unable to perform splice_in over range   ",f6.3," - ",f6.3," um")',l0,l1

  if (mode ne 1) then c=[1.0,0.0] else c=[0.0,0.0]

endelse

return,c
end

