FUNCTION splice_in2,la,fa,ea,lb,fb,eb,l0,l1,l2,l3,MODE=mode

; version 4.1
; 18 Apr 02 - fix minor bug
;  5 Apr 02 - to calculate errors from spec./phot. error vectors (v 4.1)
; 29 Mar 02 - to return errors in flux units (v 4.0)
;  7 Mar 02 - for additive corrections, now returns error as ratio (v 3.0)
;  4 Mar 02 - diagnostic changes only
; 30 Jan 02 - modified splice_in.pro to allow gap in splicing range
;             splicing will take place in ranges l0-l1 and l2-l3
; 24 Jan 02 - fix uncertainty calculation (KK)
; 18 Dec 01 - calculate uncertainty in normalization (KK)
;  2 Oct 01 - support very long vectors
;  8 Aug 01 - to correctly check for data-in-range problems
;  7 Aug 01 - to write error message to log file
; 23 Jul 01 - to handle non-standard 1-D array format
;             find multiplicative or additive correction
;             and check for no data in range
;             no longer applies normalization to data
; 20 Jul 01 - to return normalization constant
; 20 Oct 97 - created as splice_inx.pro
;
; finds multiplicativesplices together two spectral segments
; fits second to first
; uses averages over input wavelength range to determine
;    multiplicative or additive correction
;    (i.e. combines functions of splice_inx.pro and splice_ina.pro
; input:
;	la,lb = spectral wavelengths
;	fa,fb = spectral fluxes (must be same length as la,lb)
;	l0 = starting wavelength of first segment
;	l1 = end of first segment
;       l2 = starting wavelength of second segment
;       l3 = end of second segment
;       mode = 0 for multiplicative correction (default)
;       mode = 1 for additive correction
; output:  array of two factors 
;       first is the normalization factor (additive or multiplicative)
;       second is the error in the first (always in Jy) 

; check mode

if (n_elements(mode) eq 0) then mode=0 ; (multiplicative default)

; sort wavelength ranges
ll=[l0,l1,l2,l3]
l=ll([sort(ll)])
l0=l(0) & l1=l(1) & l2=l(2) & l3=l(3)

; find overlap between segments and determine error condition 
;   error=0 means overlap between la and lb covers full range l0-l1
;   error=1 means la and lb overlap, but not over full range l0-l1
;           correct for this by reducing range l0-l1 to match
;   error=2 means that la and lb do not overlap in any part of range l0-l1

if (min(la) lt min(lb)) then olapmin=min(lb) else olapmin=min(la)
if (max(la) lt max(lb)) then olapmax=max(la) else olapmax=max(lb)

if (olapmax le olapmin) then begin               ; segments don't overlap 
  error=2
endif else begin                                 ; overlap possibilities
  case 1 of
    (olapmin gt l3 or olapmax lt l0)  : error=2  ; no overlap with l0-l3
    (olapmin gt l1 and olapmax lt l2) : error=2  ; overlap between 11-12
    (olapmin le l0 and olapmax ge l3) : error=0  ; everything is fine
    else                              : error=1  ; must move l0 and/or l3
  endcase
endelse

; if error=1 then 
; (1) modify overlap range if endpoint falls in l1-l2 gap
; (2)shift l0 and/or l3 and print warning message

if (error eq 1) then begin
  if (olapmin gt l1 and olapmin lt l2) then olapmin=l2
  if (olapmax gt l1 and olapmax lt l2) then olapmax=l1
  if (olapmin gt l0) then begin
     l0=olapmin                                  ; move l0 up
     if (l1 lt l0) then l1=l0                    ; move l1 up if needed
     if (l2 lt l1) then l2=l1                    ; move l2 up if needed
  endif
  if (olapmax lt l3) then begin
    l3=olapmax                                   ; move l3 down
    if (l2 gt l3) then l2=l3                     ; move l2 down if needed
    if (l1 gt l2) then l1=l2                     ; move l1 down if needed
  endif
  print,format='("splice_in2 performed over reduced range  ",f6.3," - ",f6.3," um")',l0,l3
  printf,99,format='("  splice_in2 performed over reduced range  ",f6.3," - ",f6.3," um")',l0,l3
endif

if (error lt 2) then begin

; set stops

  jastop=long(n_elements(la)-1) ; OK only if la is 1-D
  jbstop=long(n_elements(lb)-1) ; ditto
  j = long(0) & while (la(j) lt l0) do j=j+1 & ja0 = j ; ja0
  j = long(0) & while (lb(j) lt l0) do j=j+1 & jb0 = j ; jb0
  j = ja0     & while (la(j) lt l1) do j=j+1 & ja1 = j ; ja1
  j = jb0     & while (lb(j) lt l1) do j=j+1 & jb1 = j ; jb1
  j = jastop & while (la(j) gt l3) do j=j-1 & ja3 = j  ; ja3
  j = jbstop & while (lb(j) gt l3) do j=j-1 & jb3 = j  ; jb3
  j = ja3    & while (la(j) gt l2) do j=j-1 & ja2 = j  ; ja2
  j = jb3    & while (lb(j) gt l2) do j=j-1 & jb2 = j  ; jb2
  if (ja2 eq ja1) then ja2=ja1+1
  if (jb2 eq jb1) then jb2=jb1+1

; find the average flux from each band in the overlap region

  mfa = (total(fa(ja0:ja1))+total(fa(ja2:ja3)))/float((ja1-ja0+1)+(ja3-ja2+1))
  mfb = (total(fb(jb0:jb1))+total(fb(jb2:jb3)))/float((jb1-jb0+1)+(jb3-jb2+1))

; compute uncertainty in the mean using error vectors

  mea = total((fa(ja0:ja1)-mfa)^2) + total((fa(ja2:ja3)-mfa)^2)
  mea = sqrt(mea/float((ja1-ja0)+(ja3-ja2)))
  meb = total((fb(jb0:jb1)-mfb)^2) + total((fb(jb2:jb3)-mfb)^2)
  meb = sqrt(meb/float((jb1-jb0)+(jb3-jb2)))

; compute correction, put multiplicative correction error in flux units

  c=fltarr(2)
  if (mode eq 0) then begin
    c(0) = mfa/mfb 
    c(1) = sqrt( (mea/mfb)^2 + (meb*mfa/mfb^2)^2 )
  endif else if (mode eq 1) then begin
    c(0) = mfa - mfb
    c(1) = sqrt( mea^2 + meb^2) 
  endif

endif else begin 

  print,format='("unable to perform splice_in2 over range  ",f6.3," - ",f6.3," um")',l0,l1
  printf,99,format='("  unable to perform splice_in2 over range  ",f6.3," - ",f6.3," um")',l0,l1

  if (mode ne 1) then c=[1.0,0.0] else c=[0.0,0.0]

endelse

return,c
end
