;FUNCTION splice_out,la,fa,la0,la1,lb,fb,lb0,lb1,l,DEGREE=degree,MODE=mode

; version 4.2
; 26 Apr 02 - to improve logic (v 4.2)
;  5 Apr 02 - to undo change of 29 Mar 02 (multiplicative error now a ratio)
; 29 Mar 02 - modify to return errors in Jy (v 4.0)
; 15 Mar 02 - algorithm simplified, error calculations improved (v 3.0)
; 24 Jan 02 - to fix   uncertainty calculation      (KK)
; 18 Dec 01 - to calc. uncertainty in normalization (KK)
;  9 Aug 01 - to flip input wavelengths if out of order
;  7 Aug 01 - to write error message to log file
; 27 Jul 01 - to catch errors in stops and other debugging
; 24 Jul 01 - to add degree=0, special case
; 23 Jul 01 - to handle non-standard 1-D array format
;             find multiplicative or additive correction
;             and check for no data in range
;             no longer applies normalization to data
; 20 Jul 01 - to return normalization constant
;  8 Jun 01 - to try cubic fits
;  2 Jun 01 - to add plotting routines for testing
; 20 Apr 01 - to add DEGREE keyword, default = linear
; 20 Oct 97 - created
;
; finds multiplicative or additive factor to splice two spectral segmennts
; fits second to first
; fits polynomials over input wavelength ranges
;   polynomial usually a line, but can be set to quadratic
; evaluates at wavelength l to find correction
; determines multiplicative or additive correction
; input:
;	la,lb = spectral wavelengths
;	fa,fb = spectral fluxes (must be same length as la,lb)
;	la0,lb0 = starting wavelengths
;	la1,lb1 = stopping wavelengths
;	l = evaluation wavelength
;       degree = 0 for special case
;                linear fit to segment 1, avg of segment 2
;              = 1 for linear fit
;              = 2 or 3 for quadratic or cubic fit
;       mode   = 0 for multiplicative correction
;              = 1 for additive correction
; output:  array of two factors
;        first is multiplicative or additive correction
;        second is error in correction, always in flux units

; define the polynomials to be used

pro poly1,x,a,f,pder    
f=a(0) + a(1)*x
end

pro poly2,x,a,f,pder
f=a(0) + a(1)*x + a(2)*x^2
end

pro poly3,x,a,f,pder    
f=a(0) + a(1)*x+ a(2)*x^2+ a(3)*x^3
end


FUNCTION splice_out,la,fa,la0,la1,lb,fb,lb0,lb1,l,DEGREE=degree,MODE=mode

; check degree

special=0
if (n_elements(degree) eq 0) then degree=1    ; set to linear if undeclared
if (degree eq 0) then special=1               ; degree=0 is a special case
if (degree gt 3 or degree lt 1) then degree=1 ; set to linear if out of range

; make sure wavelengths are in the right order

if (la0 gt la1) then begin & temp=la0 & la0=la1 & la1=temp & endif
if (lb0 gt lb1) then begin & temp=lb0 & lb0=lb1 & lb1=temp & endif

; check to see if segments overlap wavelength ranges and set stops

error=0
if (min(la) gt la1 or max(la) lt la0) then error=1
if (min(lb) gt lb1 or max(lb) lt lb0) then error=1

if (error eq 0) then begin

; set stops

  jastop=max(size(la))-1 ; OK since la is 1-D
  jbstop=max(size(lb))-1 ; ditto
  j = 0 & while (la(j) lt la0) do j=j+1 & ja0 = j
  j = 0 & while (lb(j) lt lb0) do j=j+1 & jb0 = j
  j = jastop & while (la(j) gt la1) do j=j-1 & ja1 = j
  j = jbstop & while (lb(j) gt lb1) do j=j-1 & jb1 = j

; double variables 

  la=double(la)
  lb=double(lb)
  fa=double(fa)
  fb=double(fb)

; determine lengths of arrays

  na=n_elements(la(ja0:ja1))
  wa=fltarr(na) & wa(*)=1
  nb=n_elements(lb(jb0:jb1))
  wb=fltarr(nb) & wb(*)=1

; define polynomials and variables for curve-fitting calls

  case degree of
    1: begin
         fn='poly1' & a1=[0,1]     & ca=a1 & cb=a1
       end
    2: begin
         fn='poly2' & a2=[0,1,1]   & ca=a2 & cb=a2
       end
    3: begin
         fn='poly3' & a3=[0,1,1,1] & ca=a3 & cb=a3
       end
  endcase

; fit polynomials to data from two segments

  yfita=curvefit(la(ja0:ja1),fa(ja0:ja1),wa,ca,cae,$
    function_name=fn, /noderiv, chisq=c2a)
  yfitb=curvefit(lb(jb0:jb1),fb(jb0:jb1),wb,cb,cbe,$
    function_name=fn, /noderiv, chisq=c2b)

; evaluate polynomial fits at appropriate wavelength, find errors

  case degree of
    1: begin
         value_a=ca(0)+ca(1)*l
         sig_a=cae(0)
         if (special eq 0) then begin
           value_b=cb(0)+cb(1)*l             ; the usual linear fit
           sig_b=cbe(0)
         endif else begin
           value_b=segmentavg(lb,fb,lb0,lb1) ; special case:  avg of b
           sig_b=sqrt(total( (fb(jb0:jb1)-value_b)^2 ) /nb)
           yfitb(*)=value_b                  ; if I need to plot it
         endelse
       end
    2: begin
         value_a=ca(0)+ca(1)*l+ca(2)*l*l
         value_b=cb(0)+cb(1)*l+cb(2)*l*l
         sig_a=cae(0)
         sig_b=cbe(0)
       end
    3: begin
         value_a=ca(0)+ca(1)*l+ca(2)*l^2+ca(3)*l^3
         value_b=cb(0)+cb(1)*l+cb(2)*l^2+cb(3)*l^3
         sig_a=cae(0)
         sig_b=cbe(0)
       end
  endcase

endif else begin
  print,format='("unable to perform splice_out centered at ",f6.3)',l
  print,format='("  data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',min(la),max(la),la0,la1
  print,format='("  data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',min(lb),max(lb),lb0,lb1
  printf,99,format='("  unable to perform splice_out centered at ",f6.3)',l
  printf,99,format='("    data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',min(la),max(la),la0,la1
  printf,99,format='("    data range ",f6.3,"-",f6.3,", splice range ",f6.3,"-",f6.3)',min(lb),max(lb),lb0,lb1
endelse

; determine correction at evaluation wavelength

c=fltarr(2) & c(*)=1.0
if (mode ne 1) then begin
    if (error eq 0) then begin 
        c(0) = value_a/value_b 
        c(1) = sqrt( (sig_a/value_b)^2 + (sig_b*value_a/value_b^2)^2 )
    endif else begin
        c(*)=1.0
    endelse
endif else begin
    if (error eq 0) then begin
        c(0)= value_a-value_b   
        c(1)= sqrt(sig_a^2 + sig_b^2)
    endif else begin 
        c(*)=0.0
    endelse
endelse

return,c
end
